package cn.com.brilliance.doc.service.impl;

import cn.com.brilliance.doc.config.DocBean;
import cn.com.brilliance.doc.service.WordTemplateService;
import com.deepoove.poi.XWPFTemplate;
import fr.opensagres.xdocreport.document.json.JSONObject;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import fr.opensagres.xdocreport.document.IXDocReport;
import fr.opensagres.xdocreport.document.registry.XDocReportRegistry;
import fr.opensagres.xdocreport.template.IContext;
import fr.opensagres.xdocreport.template.TemplateEngineKind;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.*;
import java.util.Map;
import java.util.UUID;

/**
 * @author weiguanying
 * @date 2023-02-20
 * @description word-docx模板生成docx并返回文件名称
 */
@Service
public class WordTemplateServiceImpl implements WordTemplateService {
    private static final Logger logger = LoggerFactory.getLogger(WordTemplateServiceImpl.class);

    @Autowired
    private DocBean docBean;

    public String docxTemplateToDocx(String templateName) {
        InputStream ins = null;
        OutputStream out = null;
        String templatesPath = docBean.getTemplatesPath() + templateName;
        try {

            //获取docx模板，模板存放路径 xxx.docx
            ins = new FileInputStream(templatesPath);
            //注册xdocreport实例并加载FreeMarker模板引擎
            IXDocReport report = XDocReportRegistry.getRegistry().loadReport(ins, TemplateEngineKind.Freemarker);
            //创建xdocreport上下文对象，用于存放具体数据
            IContext context = report.createContext();


            //创建要替换的文本变量
            context.put("name", "李梅");
            context.put("age", "20");
            context.put("school", "武汉大学");



            String FileId = UUID.randomUUID().toString();

            String newFileName = FileId + "-" + templateName;

            //输出到本地目录
            String filePath = docBean.getFilePath() + newFileName;


            out = new FileOutputStream(filePath);
            // 将前端参数写入文件输出流并保存为docx文件
            report.process(context, out);
            // 返回根据docx模板生产得docx文件
            return newFileName;

        } catch (Exception e) {
            logger.info("生成word-docx发生异常", e);
        } finally {
            try {
                if (ins != null) {
                    ins.close();
                }
                if (out != null) {
                    out.close();
                }
            } catch (IOException e) {
                logger.info("文件流关闭失败", e);
            }
        }
        return null;
    }

    /**
     *
     * 基于poi-tl模板引擎生成docx模板文件为docx
     * @param paramMap 模板文件替换参数
     * @return
     */
    public  String poiCreateWord(Map<String, Object> paramMap) {
        String templatePath = docBean.getTemplatesPath() + paramMap.get("templateName");
        String targetPath = docBean.getFilePath() + paramMap.get("fileName");
        // 读取模板渲染参数
        XWPFTemplate template = XWPFTemplate.compile(templatePath).render(paramMap);
        try {
            // 将模板参数写入路径
            template.writeToFile(targetPath);
            template.close();
        } catch (Exception e) {
            e.printStackTrace();
            logger.error("模板文件解析失败");
        }
        return targetPath;
    }

}
